class DcsConfigsController < RxModule::ApplicationController
  class JobCallMissingJobNumber < StandardError; end

  def self.set_target_before
    super + [:job_call, :ping, :force_reset]
  end

  # Since the LMS config is part of the settings, no point in having it's own permissions
  model_data_store(DcsConfig)

  def job_call
    params = job_call_params

    raise JobCallMissingJobNumber unless params[:job]

    resp = DcsClientManager.instance.execute_job_call_for_connection(@model, params)

    @model.save_call_trace(params[:job], resp)

    # NOTE: these payloads are used by other modules and external systems, modify with care.
    if resp[:success]
      # SUCCESSFUL RESPONSE PAYLOAD
      # {
      #   success: true
      #   source: 'RxCnC'
      #   tags: [
      #     left: {
      #       'tag1' : 'value1'
      #       ...
      #     },
      #     right: {
      #       'tag2' : 'value2'
      #       ...
      #     }
      #   ]
      # }
      render json: { success: true, source: @model.resolved_name, tags: resp[:tags] }, status: :ok
    else
      # FAILED RESPONSE PAYLOAD
      # {
      #   success: false
      #   source: 'RxCnC'
      #   error: {
      #      message: string,  - Human Readable error
      #      timeout: boolean,   - True on socket timeout
      #      failToFetch: boolean,  - True of connection issue (broken pipe, failed to connect, etc.)
      #      jobDoesNotExist: boolean  - True on job not found (STATUS != 0)
      #   }
      # }
      render json: { success: false, source: @model.resolved_name, error: resp[:error] }, status: :unprocessable_entity
    end
  end

  def ping
    success = DcsClientManager.instance.execute_ping_for_connection(@model)
    render json: {success: success}, status: :ok
  end

  def force_reset
    Rails::logger.warn "Received FORCE RESET for connection #{@model.resolved_name}!"
    success = DcsClientManager.instance.force_reset_for_connection(@model)
    render json: {success: success}, status: :ok
  end

  def list
    list = DcsConfig.all.map do |config|
      {
        label: "#{config.name} (#{config.description})",
        value: config.id
      }
    end

    render json: {options: list}, status: :ok
  end

  private

  def job_call_params
    params.fetch(:job_call).permit(:job, :tags => [])
  end

  def model_params
    params.fetch(:dcs_config).permit(:name, :description, :ip, :port, :device_type, :retry_on_fail_connection, :connection_timeout, :max_open_connections, :delay_on_socket_open, :hold_connection_open, :keep_call_trace)
  end
end